@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: FileSelector.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: FileSelector.py 347 2008-01-24 00:43:09Z camrdale-guest $
 
 """Enable the selective downloading of files within a torrent.
 
@@ -111,7 +111,17 @@ class FileSelector:
         cur_piece = 0
         for file, length in files:
             if not length:
-                self.filepieces.append(())
+                # Assign any zero-length pieces to the empty file
+                start_piece = cur_piece
+                for cur_piece in xrange(start_piece,len(piece_lengths)):
+                    if piece_lengths[cur_piece] > 0:
+                        break
+                end_piece = cur_piece-1
+                if piece_lengths[cur_piece] == 0:
+                    # Special case if the last file in a torrent is empty
+                    end_piece = cur_piece
+                pieces = range(start_piece,end_piece+1)
+                self.filepieces.append(tuple(pieces))
             else:
                 total += length
                 start_piece = cur_piece
@@ -133,8 +143,8 @@ class FileSelector:
     def init_priorities(self, init_priority):
         """Initialize the priorities of all the files from the unpickled state.
         
-        @type new_priority: C{list} of C{int}
-        @param new_priority: the new file priorities
+        @type init_priority: C{list} of C{int}
+        @param init_priority: the new file priorities
         @rtype: C{boolean}
         @return: whether the initialization was successful
         
@@ -153,7 +163,7 @@ class FileSelector:
             for f in xrange(self.numfiles):
                 if init_priority[f] < 0:
                     self.storage.disable_file(f)
-                else:
+                elif self.files[f][1] > 0:
                     self.storage.enable_file(f)
             self.storage.reset_file_status()
             self.init_priority = init_priority
@@ -167,14 +177,18 @@ class FileSelector:
         """Unpickle the previously saved state.
         
         Data is in the format::
-            d['priority'] = [file #1 priority, file #2 priority, ...]
+            d['priority'] = {file name: priority, file name: priority, ...}
         
-        It is a list of download priorities for each file. The priority may be::
+        It is a dictionary of file names and their corrseponding download
+        priorities for each file. The priority must be::
             -1 -- download disabled
              0 -- highest
              1 -- normal
              2 -- lowest
 
+        Any files not listed are assumed to have a priority of -1 (download
+        disabled).
+        
         Also see Storage.pickle and StorageWrapper.pickle for additional keys.
 
         @type d: C{dictionary}
@@ -183,8 +197,24 @@ class FileSelector:
         """
         
         if d.has_key('priority'):
-            if not self.init_priorities(d['priority']):
+            try:
+                saved_files = d['priority']
+    
+                # Create the new file priorities list from the saved ones
+                new_priority = []
+                for file, length in self.files:
+                    if not length or not file:
+                        # Empty files are not downloaded
+                        new_priority.append(-1)
+                    else:
+                        new_priority.append(saved_files.get(file, -1))
+
+                if not self.init_priorities(new_priority):
+                    return
+            except:
+                logger.exception('Error unpickling file priority cache')
                 return
+            
         pieces = self.storage.unpickle(d)
         init_piece_priority = self._get_piece_priority_list(self.init_priority)
         self.storagewrapper.reblock([i == -1 for i in init_piece_priority])
@@ -242,7 +272,7 @@ class FileSelector:
         new_disabled = [p == -1 for p in new_priority]
         data_to_update = []
         for f in xrange(self.numfiles):
-            if new_disabled[f] != old_disabled[f]:
+            if new_disabled[f] != old_disabled[f] and self.files[f][1] > 0:
                 data_to_update.extend(self.storage.get_piece_update_list(f))
         buffer = []
         for piece, start, length in data_to_update:
@@ -258,7 +288,7 @@ class FileSelector:
                 if new_disabled[f] and not old_disabled[f]:
                     self.storage.disable_file(f)
                     files_updated = True
-                if old_disabled[f] and not new_disabled[f]:
+                if old_disabled[f] and not new_disabled[f] and self.files[f][1] > 0:
                     self.storage.enable_file(f)
                     files_updated = True
         except (IOError, OSError), e:
@@ -294,7 +324,7 @@ class FileSelector:
         """
         l = [-1] * self.numpieces
         for f in xrange(self.numfiles):
-            if file_priority_list[f] == -1:
+            if file_priority_list[f] == -1 or self.files[f][1] == 0:
                 continue
             for i in self.filepieces[f]:
                 if l[i] == -1:
@@ -467,7 +497,16 @@ class FileSelector:
         
         """
         
-        d = {'priority': self.priority}
+        files = {}
+        for i in xrange(len(self.priority)):
+            if not self.files[i][1]:    # length == 0
+                continue
+            if self.priority[i] == -1:
+                continue
+            files[self.files[i][0]] = self.priority[i]
+
+        logger.info('Found %d files that need priority pickling' % len(files.keys()))
+        d = {'priority': files}
         try:
             s = self.storage.pickle()
             sw = self.storagewrapper.pickle()
@@ -476,5 +515,5 @@ class FileSelector:
             for k in sw.keys():
                 d[k] = sw[k]
         except (IOError, OSError):
-            pass
+            logger.exception('An error occurred in pickling the data')
         return d
