@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: HTTPDownloader.py 383 2008-06-20 03:30:24Z camrdale-guest $
+# $Id: HTTPDownloader.py 400 2008-07-19 23:52:10Z camrdale-guest $
 
 """Manage downloading pieces over HTTP.
 
@@ -180,8 +180,8 @@ class SingleDownload:
         """
 
         # Only use HTTP to download pieces not found in peers
-        if self.downloader.picker.numhaves[index] > 0:
-            return False
+        #if self.downloader.picker.numhaves[index] > 0:
+        #    return False
 
         if self.endflag:
             return self.downloader.storage.do_I_have_requests(index)
