@@ -3,7 +3,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: Rerequester.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: Rerequester.py 341 2008-01-23 00:16:36Z camrdale-guest $
 
 """Communicate with a tracker.
 
@@ -178,7 +178,7 @@ class Rerequester:
                   sched, externalsched, errorfunc, connect,
                   howmany, amount_left, up, down, upratefunc, downratefunc,
                   doneflag, unpauseflag = fakeflag(True),
-                  seededfunc = None, force_rapid_update = False ):
+                  seededfunc = None, force_rapid_update = False, name = ''):
         """Initialize the instance.
         
         @type port: C{int}
@@ -222,6 +222,9 @@ class Rerequester:
         @type force_rapid_update: C{boolean}
         @param force_rapid_update: whether to do quick tracker updates when 
             requested (optional, defaults to False)
+        @type name: C{string}
+        @param name: the name of the torrent to send to the tracker
+            (optional, defaults to not sending anything)
         
         """
 
@@ -277,6 +280,8 @@ class Rerequester:
             self.url += '&seed_id='+quote(seed_id)
         if self.seededfunc:
             self.url += '&check_seeded=1'
+        if name:
+            self.url += '&name='+quote(name)
 
         self.last = None
         self.trackerid = None
