@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: StorageWrapper.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: StorageWrapper.py 335 2008-01-19 22:05:46Z camrdale-guest $
 
 """Manage the storage of data at the piece level.
 
@@ -396,7 +396,7 @@ class StorageWrapper:
         self.bgalloc_active = False
         self.total_length = storage.get_total_length()
         self.amount_left = self.total_length
-        if self.total_length <= self.datalength - self.piece_sizes[-1]:
+        if self.total_length < self.datalength:
             raise ValueError, 'bad data in responsefile - total too small'
         if self.total_length > self.datalength:
             raise ValueError, 'bad data in responsefile - total too big'
@@ -532,6 +532,8 @@ class StorageWrapper:
             assert not got.has_key(v)
             got[v] = 1
         for i in xrange(len(self.hashes)):
+            if self.piece_sizes[i] == 0:
+                continue
             if self.places.has_key(i):  # restored from pickled
                 self.check_targets[self.hashes[i]] = []
                 if self.places[i] == i:
@@ -864,6 +866,7 @@ class StorageWrapper:
         """
         
         length = self._piecelen(index)
+        assert length > 0
         l = []
         x = 0
         while x + self.request_size < length:
@@ -929,7 +932,7 @@ class StorageWrapper:
         Random bits are removed from the bitfield and added to the list of haves.
         
         @rtype: (C{string}, C{list} of C{int})
-        @return: the incomplete bitfiled as a binary string, and the haves to
+        @return: the incomplete bitfield as a binary string, and the haves to
             complete it
         
         """
@@ -1263,6 +1266,7 @@ class StorageWrapper:
         """
         
         assert not self.have[index]
+        assert self.piece_sizes[index] > 0
         
         if not self.places.has_key(index):
             while self._clear_space(index):
@@ -1567,10 +1571,7 @@ class StorageWrapper:
         
         Pickled data format::
     
-            d['pieces'] = either a string containing a bitfield of complete pieces,
-                        or the numeric value "1" signifying a seed.  If it is
-                        a seed, d['places'] and d['partials'] should be empty
-                        and needn't even exist.
+            d['pieces'] = a string containing a bitfield of complete pieces
             d['partials'] = [ piece, [ offset, length... ]... ]
                         a list of partial data that had been previously
                         downloaded, plus the given offsets.  Adjacent partials
@@ -1592,8 +1593,6 @@ class StorageWrapper:
         
         """
         
-        if self.have.complete():
-            return {'pieces': 1}
         pieces = Bitfield(len(self.hashes))
         places = []
         partials = []
@@ -1650,26 +1649,20 @@ class StorageWrapper:
         restored_partials = []
 
         try:
-            if data['pieces'] == 1:     # a seed
-                assert not data.get('places',None)
-                assert not data.get('partials',None)
-                have = Bitfield(len(self.hashes))
-                for i in xrange(len(self.hashes)):
+            have = Bitfield(len(self.hashes))
+            old_have = Bitfield(len(self.hashes), data['pieces'])
+            for i in xrange(len(self.hashes)):
+                if old_have[i] and self.piece_sizes[i] > 0:
                     have[i] = True
-                assert have.complete()
-                _places = []
-                _partials = []
-            else:
-                have = Bitfield(len(self.hashes), data['pieces'])
-                _places = data['places']
-                assert len(_places) % 2 == 0
-                _places = [_places[x:x+2] for x in xrange(0,len(_places),2)]
-                _partials = data['partials']
-                assert len(_partials) % 2 == 0
-                _partials = [_partials[x:x+2] for x in xrange(0,len(_partials),2)]
+            _places = data['places']
+            assert len(_places) % 2 == 0
+            _places = [_places[x:x+2] for x in xrange(0,len(_places),2)]
+            _partials = data['partials']
+            assert len(_partials) % 2 == 0
+            _partials = [_partials[x:x+2] for x in xrange(0,len(_partials),2)]
                 
             for index, place in _places:
-                if place not in valid_places:
+                if place not in valid_places or self.piece_sizes[index] == 0:
                     continue
                 assert not got.has_key(index)
                 assert not got.has_key(place)
@@ -1696,6 +1689,8 @@ class StorageWrapper:
             for index, plist in _partials:
                 assert not dirty.has_key(index)
                 assert not have[index]
+                if self.piece_sizes[index] == 0:
+                    continue
                 if not places.has_key(index):
                     if index not in valid_places:
                         continue
