@@ -1,7 +1,7 @@
 # Written by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: HTTPCache.py 268 2007-08-18 23:45:45Z camrdale-guest $
+# $Id: HTTPCache.py 382 2008-06-20 03:27:06Z camrdale-guest $
 
 """Manage an HTTP download cache.
 
@@ -27,6 +27,9 @@ from os.path import join, split, getmtime, getsize, exists
 from os import utime, makedirs, listdir
 from time import strftime, strptime, gmtime
 from calendar import timegm
+from bz2 import BZ2Decompressor
+from zlib import decompressobj, MAX_WBITS
+from gzip import FCOMMENT, FEXTRA, FHCRC, FNAME, FTEXT
 import logging
 
 logger = logging.getLogger('DebTorrent.HTTPCache')
@@ -43,24 +46,35 @@ class CacheRequest:
     @ivar path: the server and path to download
     @type func: C{method}
     @ivar func: the method to call when the download completes
+    @type filename: C{string}
+    @ivar filename: the file to save the downloaded data to
     @type response: (C{int}, C{string}, C{dictionary}, C{string})
     @ivar response: the HTTP status code, status message, headers, and
         downloaded data
     
     """
     
-    def __init__(self, path, func):
+    def __init__(self, path, func, filename, decompress = False):
         """Initialize the instance.
         
         @type path: C{list} of C{string}
         @param path: the server and path to download
         @type func: C{method}
         @param func: the method to call when the download completes
+        @type filename: C{string}
+        @param filename: the file to save the downloaded data to
+        @type decompress: C{boolean}
+        @param decompress: whether to decompress the file while downloading it
+            (optional, defaults to not decompressing)
         
         """
         
         self.path = path
         self.func = func
+        self.filename = filename
+        self.decompress = decompress
+        self.gzfile = None
+        self.bz2file = None
         self.response = None
         
     def save_response(self, r):
@@ -71,8 +85,132 @@ class CacheRequest:
         
         """
         
-        self.response = (r.status, r.reason, dict(r.getheaders()), r.read())
+        logger.debug('processing response: %s', self.filename)
+        if r.status == 200:
+            # Create the directory for the new file
+            new_dir = split(self.filename)[0]
+            if new_dir != '' and not exists(new_dir):
+                makedirs(new_dir)
+            
+            # Write the new file
+            f = open(self.filename, 'wb')
+            if self.decompress and self.filename.endswith(".gz"):
+                self.gzheader = True
+                self.gzfile = open(self.filename[:-3], 'wb')
+                self.gzdec = decompressobj(-MAX_WBITS)
+            elif self.decompress and self.filename.endswith(".bz2"):
+                self.bz2file = open(self.filename[:-4], 'wb')
+                self.bz2dec = BZ2Decompressor()
+
+            data = r.read(4096)
+            while len(data) > 0:
+                f.write(data)
+                if self.gzfile:
+                    # Decompress the zlib portion of the file
+                    if self.gzheader:
+                        # Remove the gzip header junk
+                        self.gzheader = False
+                        new_data = self._remove_gzip_header(data)
+                        dec_data = self.gzdec.decompress(new_data)
+                    else:
+                        dec_data = self.gzdec.decompress(data)
+                    self.gzfile.write(dec_data)
+                elif self.bz2file:
+                    # Decompress the bz2 file
+                    dec_data = self.bz2dec.decompress(data)
+                    self.bz2file.write(dec_data)
+                    
+                data = r.read(4096)
+                
+            f.close()
+            r.close()
+            
+            # Set the modified time (on error use current time which should work)
+            try:
+                mtime = timegm(strptime(r.getheader('last-modified'), time_format + ' %Z'))
+                times = (mtime, mtime)
+                utime(self.filename, times)
+            except:
+                logger.exception('Failed to set the cache time for the file')
+                times = None
+
+            # Close the decompressed file
+            if self.gzfile:
+                # Finish the decompression 
+                data_dec = self.gzdec.flush()
+                self.gzfile.write(data_dec)
+                self.gzfile.close()
+                self.gzfile = None
+
+                # Set the modified time (on error use current time which should work)
+                if times:
+                    try:
+                        utime(self.filename[:-3], times)
+                    except:
+                        logger.exception('Failed to set the cache time for the decompressed file')
+            elif self.bz2file:
+                self.bz2file.close()
+                self.bz2file = None
+
+                # Set the modified time (on error use current time which should work)
+                if times:
+                    try:
+                        utime(self.filename[:-4], times)
+                    except:
+                        logger.exception('Failed to set the cache time for the decompressed file')
+        else:
+            data = r.read()
         
+        headers = {'Server': VERSION}
+
+        # Use the headers we want
+        if exists(self.filename):
+            mtime_string = strftime(time_format + ' GMT', gmtime(getmtime(self.filename)))
+            headers['last-modified'] = mtime_string
+        
+        for k, v in r.getheaders():
+            if k in ('last-modified', 'content-type'):
+                headers[k] = v
+                
+        self.response = (r.status, r.reason, headers, data)
+        
+    def _remove_gzip_header(self, data):
+        """Remove the gzip header from the zlib compressed data."""
+        # Read, check & discard the header fields
+        if data[:2] != '\037\213':
+            raise IOError, 'Not a gzipped file'
+        if ord(data[2]) != 8:
+            raise IOError, 'Unknown compression method'
+        flag = ord(data[3])
+        # modtime = self.fileobj.read(4)
+        # extraflag = self.fileobj.read(1)
+        # os = self.fileobj.read(1)
+
+        skip = 10
+        if flag & FEXTRA:
+            # Read & discard the extra field
+            xlen = ord(data[10])
+            xlen = xlen + 256*ord(data[11])
+            skip = skip + 2 + xlen
+        if flag & FNAME:
+            # Read and discard a null-terminated string containing the filename
+            while True:
+                if not data[skip] or data[skip] == '\000':
+                    break
+                skip += 1
+            skip += 1
+        if flag & FCOMMENT:
+            # Read and discard a null-terminated string containing a comment
+            while True:
+                if not data[skip] or data[skip] == '\000':
+                    break
+                skip += 1
+            skip += 1
+        if flag & FHCRC:
+            skip += 2     # Read & discard the 16-bit header CRC
+
+        return data[skip:]
+
     def error(self, error_msg):
         """Save an error response.
         
@@ -135,24 +273,22 @@ class CacheConnection:
             logger.exception('cannot connect to http server: '+self.server)
             self.close()
 
-    def queue(self, path, func):
+    def queue(self, req):
         """Queue a download for later starting.
         
-        @type path: C{list} of C{string}
-        @param path: the server and path to download
-        @type func: C{method}
-        @param func: the method to call when the download completes
+        @type req: L{CacheRequest}
+        @param req: the cache request object to queue
         @rtype: C{boolean}
         @return: whether the download was successfully queued
         
         """
         
-        assert path[0] == self.server
+        assert req.path[0] == self.server
         if self.closed:
             return False
 
-        logger.debug('queueing request for '+'/'.join(path))
-        self.request_queue.append(CacheRequest(path, func))
+        logger.debug('queueing request for '+'/'.join(req.path))
+        self.request_queue.append(req)
         self._run_queue()
 
         return True
@@ -223,8 +359,7 @@ class CacheConnection:
         self._run_queue()
         
         # Return the result
-        self.handler.download_complete(request.path, request.func,
-                                       request.response)
+        self.handler.download_complete(request)
         
     def auto_close(self):
         """Close the connection if it has been idle."""
@@ -242,17 +377,14 @@ class CacheConnection:
         if self.request:
             if not self.request.response:
                 self.request.error('connection closed prematurely')
-            self.handler.download_complete(self.request.path,
-                                           self.request.func,
-                                           self.request.response)
+            self.handler.download_complete(self.request)
             self.request = None
         
         # Process any waiting requests
         for request in self.request_queue:
             if not request.response:
                 request.error('connection closed prematurely')
-            self.handler.download_complete(request.path, request.func,
-                                           request.response)
+            self.handler.download_complete(request)
         del self.request_queue[:]
         
         # Remove the connection to the server
@@ -286,13 +418,16 @@ class HTTPCache:
         self.downloads = {}
         self.cachedir = cachedir
 
-    def download_get(self, path, func):
+    def download_get(self, path, func, decompress = False):
         """Create a new download from a site.
         
         @type path: C{list} of C{string}
         @param path: the server and path to download
         @type func: C{method}
         @param func: the method to call with the data when the download is complete
+        @type decompress: C{boolean}
+        @param decompress: whether to decompress the file while downloading it
+            (optional, defaults to not decompressing)
         
         """
         
@@ -300,7 +435,8 @@ class HTTPCache:
             logger.info('Opening a connection to server: '+path[0])
             self.downloads[path[0]] = CacheConnection(self, path[0])
 
-        if not self.downloads[path[0]].queue(path, func):
+        filename = self.get_filename(path)
+        if not self.downloads[path[0]].queue(CacheRequest(path, func, filename, decompress)):
             func(path, (500, 'Internal Server Error', 
                         {'Server': VERSION, 
                          'Content-Type': 'text/html; charset=iso-8859-1'},
@@ -319,56 +455,26 @@ class HTTPCache:
         assert self.downloads[server] == d
         del self.downloads[server]
 
-    def download_complete(self, path, func, r):
+    def download_complete(self, req):
         """Process the returned data from a request.
         
-        Once a download has been completed, save the downloaded file in the
-        file system. Then return the data to the callback function.
+        Once a download has been completed, return the data to the callback
+        function.
         
-        @type path: C{list} of C{string}
-        @param path: the server and path that was downloaded
-        @type func: C{method}
-        @param func: the method to call with the data
-        @type r: (C{int}, C{string}, C{dictionary}, C{string})
-        @param r: the HTTP status code, status message, headers, and downloaded data
+        @type req: L{CacheRequest}
+        @param req: the cache request object containing the response
         
         """
         
-        logger.info('download completed for: http://'+'/'.join(path))
+        logger.info('download completed for: http://'+'/'.join(req.path))
 
-        file = self.get_filename(path)
-        headers = {'Server': VERSION}
-
-        if r[0] == 200:
-            # Create the directory for the new file
-            new_dir = split(file)[0]
-            if new_dir != '' and not exists(new_dir):
-                makedirs(new_dir)
+        # Return None for the filename if the download failed
+        filename = req.filename
+        if req.response[0] not in [200, 304]:
+            filename = None
             
-            # Write the new file
-            f = open(file, 'wb')
-            f.write(r[3])
-            f.close()
-            
-            # Set the modified time (on error use current time which should work)
-            try:
-                mtime = timegm(strptime(r[2]['last-modified'], time_format + ' %Z'))
-                times = (mtime, mtime)
-                utime(file, times)
-            except:
-                logger.exception('Failed to set the cache time for the file')
-
-        # Use the headers we want
-        if exists(file):
-            mtime_string = strftime(time_format + ' GMT', gmtime(getmtime(file)))
-            headers['last-modified'] = mtime_string
-        
-        for k, v in r[2].items():
-            if k in ('last-modified', 'content-type'):
-                headers[k] = v
-        
         # Call the callback function
-        func(path, (r[0], r[1], headers, r[3]))
+        req.func(req.path, req.response, filename)
 
     def cache_get(self, path, uptodate = False, if_modified_time = ''):
         """Get the file from the cache.
@@ -387,8 +493,9 @@ class HTTPCache:
         @type if_modified_time: C{string}
         @param if_modified_time: the if-modified-since header from the request
             (optional, defaults to not checking the if-modified-time)
-        @rtype: (C{int}, C{string}, C{dictionary}, C{string})
-        @return: the HTTP status code, status message, headers, and package data
+        @rtype: (C{int}, C{string}, C{dictionary}, C{string}), C{string}
+        @return: the HTTP status code, status message, headers, and error string,
+            and the name of the file in the cache, if it was found
         
         """
         
@@ -397,7 +504,7 @@ class HTTPCache:
         # Check if the file isn't in the cache
         if not exists(file):
             logger.info('cache miss: '+file)
-            return (404, 'Not Found', {'Server': VERSION, 'Content-Type': 'text/plain', 'Pragma': 'no-cache'}, alas)
+            return (404, 'Not Found', {'Server': VERSION, 'Content-Type': 'text/plain', 'Pragma': 'no-cache'}, alas), None
         
         if uptodate:
             # Get the last modified time from the server
@@ -410,21 +517,18 @@ class HTTPCache:
             # Check if the cached data is stale
             if self.check_mtime(last_modified, file = file) > 0:
                 logger.info('cache out of date: '+file)
-                return (405, 'Method Not Allowed', {'Server': VERSION, 'Content-Type': 'text/plain', 'Pragma': 'no-cache'}, alas)
+                return (405, 'Method Not Allowed', {'Server': VERSION, 'Content-Type': 'text/plain', 'Pragma': 'no-cache'}, alas), None
 
         # Check if the request needs the data
         if if_modified_time and self.check_mtime(if_modified_time, file = file) >= 0:
             logger.info('cache up to date and so is request: '+file)
-            return (304, 'Not Modified', {'Server': VERSION, 'Pragma': 'no-cache'}, '')
+            return (304, 'Not Modified', {'Server': VERSION, 'Pragma': 'no-cache'}, ''), file
             
         # Read in the file and return the data
-        f = open(file, 'rb')
-        data = f.read()
-        f.close()
         mtime_string = strftime(time_format+' GMT', gmtime(getmtime(file)))
         
         logger.info('cache hit: '+file)
-        return (200, 'OK', {'Server': VERSION, 'Content-Type': 'text/plain', 'Last-Modified': mtime_string}, data)
+        return (200, 'OK', {'Server': VERSION, 'Content-Type': 'text/plain', 'Last-Modified': mtime_string}, ''), file
 
     def get_filename(self, path):
         """Get the file name used for this path in the cache.
