@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: download_bt1.py 381 2008-06-19 22:40:22Z camrdale-guest $
+# $Id: download_bt1.py 402 2008-07-26 00:43:37Z camrdale-guest $
 
 """Manage a single download.
 
@@ -74,7 +74,7 @@ defaults = [
         '(0 = disabled)'),
     ('log_dir', '',
         'directory to write the logfiles to (default is to use the cache directory)'),
-    ('log_level', 10,
+    ('log_level', 20,
         'level to write the logfiles at, varies from 10 (debug) to 50 (critical)'),
     # Rate limits
     ('max_upload_rate', 0,
@@ -905,7 +905,7 @@ class BT1Download:
             else:
                 self.pickled_data['resume data'] = {'priority': {}, 'files': {}}
                 must_find_files = True
-            self.pickled_data['stats'] = {'upload': 0L, 'download': 0L}
+            self.pickled_data['stats'] = {'upload': 0L, 'download': 0L, 'download_http': 0L}
             self.pickled_data['version'] = 1
 
         # Initialize the saved state it if it wasn't found
@@ -913,7 +913,7 @@ class BT1Download:
             must_find_files = True
             self.pickled_data = {}
             self.pickled_data['resume data'] = {'priority': {}, 'files': {}}
-            self.pickled_data['stats'] = {'upload': 0L, 'download': 0L}
+            self.pickled_data['stats'] = {'upload': 0L, 'download': 0L, 'download_http': 0L}
             self.pickled_data['version'] = 1
 
         enabled_files = []
@@ -1099,7 +1099,7 @@ class BT1Download:
         
         """
         
-        self.downmeasure.update_rate(x)
+        self.downmeasure.update_http_rate(x)
         self.ratemeasure.data_came_in(x)
         self.downloader.external_data_received(x)
 
@@ -1139,7 +1139,7 @@ class BT1Download:
         @return: whether the engines were started
         
         """
-        
+
         if self.doneflag.isSet():
             return False
 
@@ -1162,9 +1162,13 @@ class BT1Download:
         
         total_up = long(self.pickled_data['stats']['upload'])
         total_down = long(self.pickled_data['stats']['download'])
+        # The total_http stat isn't in files created with debtorrent-0.1.8
+        total_http = 0L;
+        if self.pickled_data['stats'].has_key('download_http'):
+            total_http = long(self.pickled_data['stats']['download_http'])
         self.upmeasure = Measure(self.config['max_rate_period'],
                             self.config['upload_rate_fudge'], saved_total = total_up)
-        self.downmeasure = Measure(self.config['max_rate_period'], saved_total = total_down)
+        self.downmeasure = Measure(self.config['max_rate_period'], saved_total = total_down, http_total = total_http)
 
         if ratelimiter:
             self.ratelimiter = ratelimiter
@@ -1308,11 +1312,9 @@ class BT1Download:
         return self.encoder
 
 
-    def shutdown(self, torrentdata = {}):
+    def shutdown(self):
         """Shutdown the running download.
         
-        @type torrentdata: C{dictionary}
-        @param torrentdata: any data that needs to be saved (pickled)
         @rtype: C{boolean}
         @return: False if a failure or exception occurred
         
@@ -1324,9 +1326,11 @@ class BT1Download:
             self.storage.close()
             self.rerequest_stopped()
         if self.fileselector and self.started:
+            torrentdata = {}
             torrentdata['version'] = 1
             torrentdata['stats'] = {'upload': self.upmeasure.get_total(),
-                                    'download': self.downmeasure.get_total()}
+                                    'download': self.downmeasure.get_total(),
+                                    'download_http': self.downmeasure.get_http_subtotal()}
             if not self.failed:
                 self.fileselector.finish()
                 torrentdata['resume data'] = self.fileselector.pickle()
