@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: launchmanycore.py 389 2008-06-22 19:23:06Z camrdale-guest $
+# $Id: launchmanycore.py 401 2008-07-26 00:36:45Z camrdale-guest $
 
 """Manage the downloading of multiple torrents in one process.
 
@@ -449,6 +449,7 @@ class LaunchMany:
             dnrate = 0.0
             upamt = 0
             dnamt = 0
+            httpdnamt = 0
             t = 0
             if d.is_dead():
                 status = 'stopped'
@@ -483,6 +484,7 @@ class LaunchMany:
                 uprate = stats['up']
                 upamt = s.upTotal
                 dnamt = s.downTotal
+                httpdnamt = s.httpDownTotal
                 size = stats['wanted']
                    
             if d.is_dead() or d.status_errtime+300 > clock():
@@ -491,7 +493,7 @@ class LaunchMany:
                 msg = ''
 
             data.append(( name, id, status, progress, peers, seeds, seedsmsg, 
-                          dist, uprate, dnrate, upamt, dnamt, size, t, msg ))
+                          dist, uprate, dnrate, upamt, dnamt, httpdnamt, size, t, msg ))
 
         return data
 
